@@ -1,4 +1,5 @@
 import logging
+import re
 from typing import Union, Any, Optional, Annotated, get_type_hints
 from uuid import uuid4
 from typing import Dict
@@ -274,11 +275,20 @@ class StateModifierNode(Runnable):
                 logger.warning(f"Failed to decode base64 value: {e}")
                 return value
         
+        def split_by_words(value, chunk_size=100):
+            words = value.split()
+            return [" ".join(words[i:i + chunk_size]) for i in range(0, len(words), chunk_size)]
+        
+        def split_by_regex(value, pattern):
+            """Splits the provided string using the specified regex pattern."""
+            return re.split(pattern, value)
 
         env = Environment()
         env.filters['from_json'] = from_json
-        env.filters['base64ToString'] = base64_to_string
-        
+        env.filters['base64_to_string'] = base64_to_string
+        env.filters['split_by_words'] = split_by_words
+        env.filters['split_by_regex'] = split_by_regex
+
         template = env.from_string(self.template)
         rendered_message = template.render(**input_data)
         result = {}
@@ -455,14 +465,11 @@ def create_graph(
                                 input_variables=node.get('input', ['messages'])))
                         elif node_type == 'agent':
                             input_params = node.get('input', ['messages'])
-                            input_mapping = {'task': {'type': 'fstring', 'value': f"{node.get('task', '')}"},
-                                                      'chat_history': {'type': 'fixed', 'value': []}}
-                            # Add 'chat_history' to input_mapping only if 'messages' is in input_params
-                            if 'messages' in input_params:
-                                input_mapping['chat_history'] = {'type': 'variable', 'value': 'messages'}
+                            input_mapping = node.get('input_mapping',
+                                                       {'messages': {'type': 'variable', 'value': 'messages'}})
                             lg_builder.add_node(node_id, FunctionTool(
                                 client=client, tool=tool,
-                                name=node['id'], return_type='dict',
+                                name=node['id'], return_type='str',
                                 output_variables=node.get('output', []),
                                 input_variables=input_params,
                                 input_mapping= input_mapping
@@ -576,11 +583,10 @@ def create_graph(
                     available_tools = [tool for tool in tools if isinstance(tool, BaseTool)]
                 
                 lg_builder.add_node(node_id, LLMNode(
-                    client=client, 
-                    prompt=node.get('prompt', {}),
-                    name=node['id'], 
+                    client=client,
+                    input_mapping=node.get('input_mapping', {'messages': {'type': 'variable', 'value': 'messages'}}),
+                    name=node['id'],
                     return_type='dict',
-                    response_key=node.get('response_key', 'messages'),
                     structured_output_dict=output_vars_dict,
                     output_variables=output_vars,
                     input_variables=node.get('input', ['messages']),
@@ -638,19 +644,11 @@ def create_graph(
             entry_point = clean_string(schema['entry_point'])
         except KeyError:
             raise ToolException("Entry point is not defined in the schema. Please define 'entry_point' in the schema.")
-        # if state.items():
-        #     state_default_node = StateDefaultNode(default_vars=set_defaults(state))
-        #     lg_builder.add_node(state_default_node.name, state_default_node)
-        #     lg_builder.set_entry_point(state_default_node.name)
-        #     lg_builder.add_conditional_edges(state_default_node.name, TransitionalEdge(entry_point))
-        for key, value in state.items():
-            if 'type' in value and 'value' in value:
-                # set default value for state variable if it is defined in the schema
-                state_default_node = StateDefaultNode(default_vars=state)
-                lg_builder.add_node(state_default_node.name, state_default_node)
-                lg_builder.set_entry_point(state_default_node.name)
-                lg_builder.add_conditional_edges(state_default_node.name, TransitionalEdge(entry_point))
-                break
+        if state.items():
+            state_default_node = StateDefaultNode(default_vars=set_defaults(state))
+            lg_builder.add_node(state_default_node.name, state_default_node)
+            lg_builder.set_entry_point(state_default_node.name)
+            lg_builder.add_conditional_edges(state_default_node.name, TransitionalEdge(entry_point))
         else:
             # if no state variables are defined, set the entry point directly
             lg_builder.set_entry_point(entry_point)
@@ -697,6 +695,7 @@ def set_defaults(d):
     type_defaults = {
         'str': '',
         'list': [],
+        'dict': {},
         'int': 0,
         'float': 0.0,
         'bool': False,
@@ -757,6 +756,8 @@ class LangGraphAgentRunnable(CompiledStateGraph):
         # Append current input to existing messages instead of overwriting
         if input.get('input'):
             current_message = input.get('input')[-1]
+            # TODO: add handler after we add 2+ inputs (filterByType, etc.)
+            input['input'] = current_message if isinstance(current_message, str) else str(current_message)
             if input.get('messages'):
                 # Ensure existing messages are LangChain objects
                 input['messages'] = [convert_dict_to_message(msg) for msg in input['messages']]
