@@ -138,7 +138,7 @@ def create_state(data: Optional[dict] = None):
         value = value['type'] if isinstance(value, dict) else value
         if key == 'messages':
             state_dict[key] = Annotated[list[AnyMessage], add_messages]
-        elif value in ['str', 'int', 'float', 'bool', 'list', 'dict', 'number']:
+        elif value in ['str', 'int', 'float', 'bool', 'list', 'dict', 'number', 'dict']:
             state_dict[key] = parse_type(value)
     logger.debug(f"Created state: {state_dict}")
     return TypedDict('State', state_dict)
