@@ -1043,11 +1043,39 @@ class JiraApiWrapper(NonCodeIndexerToolkit):
 
     def _extract_image_data(self, field_data):
         """
-        Extracts image data from general JSON response
+        Extracts image data from general JSON response.
+        Handles lists, dicts with image info, and plain strings.
         """
-        if isinstance(field_data, dict) and 'filename' in field_data and 'content' in field_data:
-            return f"!{field_data['filename']}|alt={field_data['filename']}!"
-        return str(field_data)
+        if isinstance(field_data, list):
+            return ' '.join(self._extract_image_data(item) for item in field_data)
+        if isinstance(field_data, dict):
+            if 'filename' in field_data and 'content' in field_data:
+                return f"!{field_data['filename']}|alt={field_data['filename']}!"
+            if 'content' in field_data and isinstance(field_data['content'], list):
+                result = []
+                for content_item in field_data['content']:
+                    if (
+                        isinstance(content_item, dict)
+                        and 'content' in content_item
+                        and isinstance(content_item['content'], list)
+                        and content_item['content']
+                    ):
+                        if content_item.get('type') == 'mediaSingle':
+                            media = content_item['content'][0]
+                            attrs = media.get('attrs', {})
+                            if attrs.get('type') == 'file':
+                                alt = attrs.get('alt', '')
+                                image_str = f'!{alt}|alt="{alt}"!'
+                                result.append(image_str)
+                        elif content_item.get('type') == 'paragraph':
+                            result.append(content_item['content'][0].get('text', ''))
+                        else:
+                            result.append(self._extract_image_data(content_item))
+                return '\n'.join(result)
+            return f"Unsupported format of field content."
+        if isinstance(field_data, str):
+            return field_data
+        return f"Unsupported field content type: {type(field_data)}. Expected a string, list, or dict."
 
     def get_field_with_image_descriptions(self, jira_issue_key: str, field_name: str, prompt: Optional[str] = None,
                                           context_radius: int = 500):
@@ -1082,12 +1110,7 @@ class JiraApiWrapper(NonCodeIndexerToolkit):
                 return f"Unable to find field '{field_name}' or it's empty. Available fields are: {existing_fields_str}"
 
             # Handle multiple images or non-string content
-            if isinstance(field_content, list):
-                field_content = ' '.join(map(self._extract_image_data, field_content))
-            elif isinstance(field_content, dict):
-                field_content = self._extract_image_data(field_content)
-            elif not isinstance(field_content, str):
-                return f"Unsupported field content type: {type(field_content)}. Expected a string, list, or dict."
+            field_content = self._extract_image_data(field_content)
 
             # Regular expression to find image references in Jira markup
             image_pattern = r'!([^!|]+)(?:\|[^!]*)?!'
@@ -1221,6 +1244,7 @@ class JiraApiWrapper(NonCodeIndexerToolkit):
 
             comment_author = comment.get('author', {}).get('displayName', 'Unknown')
             comment_created = comment.get('created', 'Unknown date')
+            comment_body = self._extract_image_data(comment_body)
 
             # Process the comment body by replacing image references with descriptions
             processed_body = re.sub(image_pattern,
@@ -1235,6 +1259,7 @@ class JiraApiWrapper(NonCodeIndexerToolkit):
                 "original_content": comment_body,
                 "processed_content": processed_body
             })
+        
         return processed_comments
 
     def get_comments_with_image_descriptions(self, jira_issue_key: str, prompt: Optional[str] = None, context_radius: int = 500):
@@ -1305,7 +1330,7 @@ class JiraApiWrapper(NonCodeIndexerToolkit):
 
             # Use provided JQL query or default to all issues
             if not jql:
-                jql_query = "ORDER BY updated DESC"  # Default to get all issues ordered by update time
+                jql_query = "created >= \"1970-01-01\" ORDER BY updated DESC"  # Default to get all issues ordered by update time
             else:
                 jql_query = jql
 
@@ -1419,7 +1444,7 @@ class JiraApiWrapper(NonCodeIndexerToolkit):
         if validate_query is not None:
             params["validateQuery"] = validate_query
 
-        url = self._client.resource_url("search")
+        url = self._client.resource_url("search/jql" if self.api_version == '3' else "search")
 
         while True:
             params["startAt"] = int(start)
@@ -1437,6 +1462,8 @@ class JiraApiWrapper(NonCodeIndexerToolkit):
                 break
             if not response["issues"]:
                 break
+            if len(issues) < limit:
+                break
             start += len(issues)
 
     def _process_issue_for_indexing(self, issue: dict, fields_to_index=None) -> Document:
