@@ -1,3 +1,4 @@
+from copy import deepcopy
 import os
 import re
 import tempfile
@@ -126,7 +127,7 @@ def load_file_docs(file_name=None, file_content=None, is_capture_image: bool = F
 
 def get_loader_kwargs(loader_object, file_name=None, file_content=None, is_capture_image: bool = False, page_number: int = None,
                     sheet_name: str = None, llm=None, file_path: str = None, excel_by_sheets: bool = False, prompt=None):
-    loader_kwargs = loader_object['kwargs']
+    loader_kwargs = deepcopy(loader_object['kwargs'])
     loader_kwargs.update({
         "file_path": file_path,
         "file_content": file_content,
@@ -269,7 +270,7 @@ def process_content_by_type(content, filename: str, llm=None, chunking_config=No
                 loader_kwargs.pop(LoaderProperties.PROMPT_DEFAULT.value)
                 loader_kwargs[LoaderProperties.PROMPT.value] = image_processing_prompt
             loader = loader_cls(file_path=temp_file_path, **loader_kwargs)
-            return loader.load()
+            yield from loader.load()
     finally:
         if temp_file_path and os.path.exists(temp_file_path):
             os.remove(temp_file_path)
@@ -296,7 +297,9 @@ def sanitize_for_postgres(text: str, replacement: str = "") -> str:
     return text.replace("\x00", replacement)
 
 
-def file_extension_by_chunker(chunker_name: str) -> str:
+def file_extension_by_chunker(chunker_name: str) -> str | None:
+    if not chunker_name:
+        return None
     name = chunker_name.lower()
     if name == "markdown":
         return ".md"
